@@ -1,9 +1,9 @@
-use chrono::{offset::Local, DateTime};
-use fuzzy_matcher::{skim::SkimMatcherV2, FuzzyMatcher};
+use std::{borrow::Cow, mem::swap, string::ToString, sync::LazyLock};
+
+use chrono::{DateTime, offset::Local};
+use fuzzy_matcher::FuzzyMatcher;
 use log::{error, warn};
-use memchr::memmem;
-use once_cell::sync::Lazy;
-use std::{borrow::Cow, mem::swap, string::ToString};
+use memchr::{memchr_iter, memmem};
 use tui::{
     style::{Color, Color::Rgb, Modifier, Style},
     text::{Line, Span},
@@ -11,24 +11,24 @@ use tui::{
 use unicode_width::UnicodeWidthStr;
 
 use crate::{
-    emotes::{display_emote, load_emote, overlay_emote, DownloadedEmotes, EmoteData, SharedEmotes},
+    emotes::{DownloadedEmotes, EmoteData, SharedEmotes, display_emote, load_emote, overlay_emote},
     handlers::config::{FrontendConfig, Palette, Theme},
     ui::statics::NAME_MAX_CHARACTERS,
     utils::{
         colors::{hsl_to_rgb, u32_to_color},
         emotes::{
-            get_emote_offset, UnicodePlaceholder, PRIVATE_USE_UNICODE, ZERO_WIDTH_SPACE,
-            ZERO_WIDTH_SPACE_STR,
+            PRIVATE_USE_UNICODE, UnicodePlaceholder, ZERO_WIDTH_SPACE, ZERO_WIDTH_SPACE_STR,
+            get_emote_offset,
         },
+        search::FUZZY_FINDER,
         styles::{
-            DATETIME_DARK, DATETIME_LIGHT, HIGHLIGHT_NAME_DARK, HIGHLIGHT_NAME_LIGHT, SYSTEM_CHAT,
+            DATETIME_DARK_STYLE, DATETIME_LIGHT_STYLE, HIGHLIGHT_NAME_DARK_STYLE,
+            HIGHLIGHT_NAME_LIGHT_STYLE, SEARCH_STYLE, SYSTEM_CHAT_STYLE,
         },
         text::split_cow_in_place,
     },
 };
 
-static FUZZY_FINDER: Lazy<SkimMatcherV2> = Lazy::new(SkimMatcherV2::default);
-
 pub enum TwitchToTerminalAction {
     Message(RawMessageData),
     ClearChat(Option<String>),
@@ -180,6 +180,10 @@ impl MessageData {
         emotes_set1: &DownloadedEmotes,
         emotes_set2: &DownloadedEmotes,
     ) -> (String, Vec<(Color, Color)>) {
+        if !emotes.enabled {
+            return (payload, vec![]);
+        }
+
         let cell_size = *emotes
             .cell_size
             .get()
@@ -325,9 +329,19 @@ impl MessageData {
         content: impl Into<Cow<'s, str>>,
         emotes: &mut &[(Color, Color)],
     ) -> Span<'s> {
+        #[allow(unused_variables)]
         if let Some(&(id, pid)) = emotes.first() {
             *emotes = &emotes[1..];
-            Span::styled(content, Style::default().fg(id).underline_color(pid))
+
+            #[cfg(not(target_os = "windows"))]
+            {
+                Span::styled(content, Style::default().fg(id).underline_color(pid))
+            }
+
+            #[cfg(target_os = "windows")]
+            {
+                Span::styled(content, Style::default().fg(id))
+            }
         } else {
             error!("Emote index >= emotes.len()");
             Span::raw(content)
@@ -379,8 +393,8 @@ impl MessageData {
         username_highlight: Highlight,
         emotes: &mut &[(Color, Color)],
     ) -> Vec<Span<'s>> {
-        static EMOTE_FINDER: Lazy<memmem::Finder> =
-            Lazy::new(|| memmem::Finder::new(ZERO_WIDTH_SPACE_STR));
+        static EMOTE_FINDER: LazyLock<memmem::Finder> =
+            LazyLock::new(|| memmem::Finder::new(ZERO_WIDTH_SPACE_STR));
 
         // A line contains emotes if `emotes` is not empty and `line` starts with a unicode placeholder or contains ZWS.
         if emotes.is_empty()
@@ -441,19 +455,18 @@ impl MessageData {
             Style::default()
         };
         let username_theme = match frontend_config.theme {
-            Theme::Dark => HIGHLIGHT_NAME_DARK,
-            _ => HIGHLIGHT_NAME_LIGHT,
+            Theme::Dark => *HIGHLIGHT_NAME_DARK_STYLE,
+            _ => *HIGHLIGHT_NAME_LIGHT_STYLE,
         };
         let author_theme = if self.system {
-            SYSTEM_CHAT
+            *SYSTEM_CHAT_STYLE
         } else {
             Style::default().fg(fg)
         };
         let datetime_theme = match frontend_config.theme {
-            Theme::Dark => DATETIME_DARK,
-            _ => DATETIME_LIGHT,
+            Theme::Dark => *DATETIME_DARK_STYLE,
+            _ => *DATETIME_LIGHT_STYLE,
         };
-        let search_theme = Style::default().fg(Color::Red).add_modifier(Modifier::BOLD);
 
         // All indices to highlight like a user
         let username_highlight = username_highlight
@@ -478,7 +491,7 @@ impl MessageData {
             })
             .unwrap_or_default();
 
-        let search = (&search_highlight as &[usize], search_theme);
+        let search = (&search_highlight as &[usize], *SEARCH_STYLE);
         let username = (&username_highlight as &[usize], username_theme);
 
         // Message prefix
@@ -549,6 +562,10 @@ impl MessageData {
         let mut first_line = lines.next().unwrap();
         let first_line_msg = split_cow_in_place(&mut first_line, prefix_len);
 
+        // Find all spaces in `self.payload`, to increment `next_index` when a space at the end of a line
+        // is encountered. Wrapping the message above removes spaces at eol, which makes the highlight indices off by 1.
+        let mut space_iter = memchr_iter(b' ', self.payload.as_bytes());
+
         let mut emotes = &self.emotes[..];
 
         first_row.extend(Self::build_line(
@@ -560,9 +577,20 @@ impl MessageData {
             &mut emotes,
         ));
 
+        let mut space_idx = space_iter.next();
+
         let mut rows = vec![Line::from(first_row)];
 
         rows.extend(lines.map(|line| {
+            while space_idx.is_some_and(|x| x < next_index) {
+                space_idx = space_iter.next();
+            }
+
+            if space_idx == Some(next_index) {
+                next_index += 1;
+                space_idx = space_iter.next();
+            }
+
             Line::from(Self::build_line(
                 line,
                 &mut next_index,
@@ -579,6 +607,7 @@ impl MessageData {
 
 #[derive(Debug, Copy, Clone)]
 pub struct DataBuilder<'conf> {
+    #[allow(dead_code)]
     pub datetime_format: &'conf str,
 }
 
@@ -626,7 +655,12 @@ impl<'conf> DataBuilder<'conf> {
 }
 
 #[cfg(test)]
+#[cfg(not(target_os = "windows"))]
 mod tests {
+    use std::{collections::BTreeMap, rc::Rc};
+
+    use crate::emotes::Emotes;
+
     use super::*;
 
     #[test]
@@ -791,8 +825,9 @@ mod tests {
         let emote_w_2 = UnicodePlaceholder::new(2).string();
         let emote_w_3 = UnicodePlaceholder::new(3).string();
 
-        let line =
-        format!("foo{ZERO_WIDTH_SPACE}{emote_w_3}{ZERO_WIDTH_SPACE}{emote_w_1}{ZERO_WIDTH_SPACE}bar baz{ZERO_WIDTH_SPACE}{emote_w_2}");
+        let line = format!(
+            "foo{ZERO_WIDTH_SPACE}{emote_w_3}{ZERO_WIDTH_SPACE}{emote_w_1}{ZERO_WIDTH_SPACE}bar baz{ZERO_WIDTH_SPACE}{emote_w_2}"
+        );
 
         let (spans, _, emotes) = assert_build_line(
             &line,
@@ -859,8 +894,9 @@ mod tests {
         let emote_w_3 = UnicodePlaceholder::new(3).string();
 
         // Line corresponds to "foo{EMOTE3}{EMOTE1}bar {EMOJI} baz{EMOTE2}".
-        let line =
-            format!("foo{ZERO_WIDTH_SPACE}{emote_w_3}{ZERO_WIDTH_SPACE}{emote_w_1}{ZERO_WIDTH_SPACE}bar \u{1f7ea} baz{ZERO_WIDTH_SPACE}{emote_w_2}");
+        let line = format!(
+            "foo{ZERO_WIDTH_SPACE}{emote_w_3}{ZERO_WIDTH_SPACE}{emote_w_1}{ZERO_WIDTH_SPACE}bar \u{1f7ea} baz{ZERO_WIDTH_SPACE}{emote_w_2}"
+        );
 
         // "ba" in "bar" will be highlighted with the username highlight.
         // " b" in " baz" will be highlighted with the search highlight.
@@ -906,4 +942,56 @@ mod tests {
             ]
         );
     }
+
+    #[test]
+    fn build_vec_with_wraps_and_highlights() {
+        let raw_message = RawMessageData::new(
+            "foo".to_string(),
+            None,
+            false,
+            "foo bar baz".to_string(),
+            BTreeMap::new(),
+            None,
+            false,
+        );
+
+        let emotes = Rc::new(Emotes::new(false));
+        let data = MessageData::from_twitch_message(raw_message, &emotes);
+
+        let frontendconfig = FrontendConfig {
+            show_datetimes: false,
+            ..FrontendConfig::default()
+        };
+
+        let lines = data.to_vec(&frontendconfig, 13, Some("bar"), None);
+
+        assert_eq!(
+            lines,
+            vec![
+                Line::from(vec![
+                    Span::styled("foo", data.hash_username(&Palette::Pastel)),
+                    Span::raw(": "),
+                    Span::raw("foo")
+                ]),
+                Line::from(vec![
+                    Span::styled(
+                        "b",
+                        Style::default().fg(Color::Red).add_modifier(Modifier::BOLD)
+                    ),
+                    Span::styled(
+                        "a",
+                        Style::default().fg(Color::Red).add_modifier(Modifier::BOLD)
+                    ),
+                    Span::styled(
+                        "r",
+                        Style::default().fg(Color::Red).add_modifier(Modifier::BOLD)
+                    ),
+                    Span::raw(" "),
+                    Span::raw("b"),
+                    Span::raw("a"),
+                    Span::raw("z")
+                ])
+            ]
+        );
+    }
 }
