@@ -1,12 +1,15 @@
 use tui::{
+    Frame,
     layout::{Alignment, Rect},
     style::{Color, Style},
-    terminal::Frame,
     text::{Line, Span},
-    widgets::{block::Title, Block, Borders, Clear, Paragraph},
+    widgets::{Block, Borders, Clear, Paragraph},
 };
 
-use crate::ui::components::Component;
+use crate::{
+    ui::components::Component,
+    utils::styles::{NO_COLOR, TEXT_DARK_STYLE},
+};
 
 #[derive(Debug, Clone)]
 pub struct ErrorWidget {
@@ -26,14 +29,14 @@ impl ErrorWidget {
         self.focused
     }
 
-    pub fn toggle_focus(&mut self) {
+    pub const fn toggle_focus(&mut self) {
         self.focused = !self.focused;
     }
 }
 
 impl Component for ErrorWidget {
     fn draw(&mut self, f: &mut Frame, area: Option<Rect>) {
-        let r = area.map_or_else(|| f.size(), |a| a);
+        let r = area.map_or_else(|| f.area(), |a| a);
 
         let paragraph = Paragraph::new(
             self.message
@@ -44,10 +47,14 @@ impl Component for ErrorWidget {
         .block(
             Block::default()
                 .borders(Borders::ALL)
-                .border_style(Style::default().fg(Color::Red))
-                .title(Title::from("[ ERROR ]").alignment(Alignment::Center)),
+                .border_style(if *NO_COLOR {
+                    Style::default()
+                } else {
+                    Style::default().fg(Color::Red)
+                })
+                .title_top(Line::from("[ ERROR ]").centered()),
         )
-        .style(Style::default().fg(Color::White))
+        .style(*TEXT_DARK_STYLE)
         .alignment(Alignment::Center);
 
         f.render_widget(Clear, r);
