@@ -1,15 +1,15 @@
 use tui::{
+    Frame,
     layout::Rect,
     style::{Color, Modifier, Style},
     symbols::DOT,
     text::Line,
     widgets::{Block, Tabs},
-    Frame,
 };
 
 use crate::{
     handlers::{config::SharedCompleteConfig, state::State},
-    utils::text::capitalize_first_char,
+    utils::{styles::STATE_TABS_STYLE, text::capitalize_first_char},
 };
 
 const TABS_TO_RENDER: [State; 3] = [State::Dashboard, State::Normal, State::Help];
@@ -20,7 +20,7 @@ pub struct StateTabsWidget {
 }
 
 impl StateTabsWidget {
-    pub fn new(config: SharedCompleteConfig) -> Self {
+    pub const fn new(config: SharedCompleteConfig) -> Self {
         Self { _config: config }
     }
 
@@ -32,7 +32,7 @@ impl StateTabsWidget {
 
         let tabs = Tabs::new(tab_titles)
             .block(Block::default())
-            .style(Style::default().fg(Color::Gray).add_modifier(Modifier::DIM))
+            .style(*STATE_TABS_STYLE)
             .highlight_style(
                 Style::default()
                     .fg(Color::Yellow)
